searchData={"items":[{"type":"task","doc":"Generate an OpenID Connect provider configuration worker\n\nAdds an `Oidcc.ProviderConfiguration.Worker` to your application and\nconfigures it via the `runtime.exs` configuration file.","title":"mix oidcc.gen.provider_configuration_worker","ref":"Mix.Tasks.Oidcc.Gen.ProviderConfigurationWorker.html"},{"type":"task","doc":"```bash\nmix oidcc.gen.provider_configuration_worker \\\n  --name MyApp.OpenIDProvider \\\n  --issuer https://accounts.google.com \n```","title":"Example - mix oidcc.gen.provider_configuration_worker","ref":"Mix.Tasks.Oidcc.Gen.ProviderConfigurationWorker.html#module-example"},{"type":"task","doc":"* `--name` or `-n` - The name of the provider configuration worker\n* `--issuer` or `-i` - The issuer of the provider","title":"Options - mix oidcc.gen.provider_configuration_worker","ref":"Mix.Tasks.Oidcc.Gen.ProviderConfigurationWorker.html#module-options"},{"type":"module","doc":"OpenID Connect High Level Interface","title":"oidcc","ref":"oidcc.html"},{"type":"module","doc":"```erlang\n{ok, Pid} =\n  oidcc_provider_configuration_worker:start_link(#{\n    issuer => <<\"https://accounts.google.com\">>,\n    name => {local, google_config_provider}\n  }).\n```\n\n(or via a `m:supervisor`)\n\nSee `m:oidcc_provider_configuration_worker` for details","title":"Setup - oidcc","ref":"oidcc.html#module-setup"},{"type":"module","doc":"* `max_clock_skew` (default `0`) - Maximum allowed clock skew for JWT\n  `exp` / `nbf` validation, in seconds","title":"Global Configuration - oidcc","ref":"oidcc.html#module-global-configuration"},{"type":"function","doc":"Retrieve Client Credential Token.\n\nSee https://datatracker.ietf.org/doc/html/rfc6749#section-1.3.4.","title":"oidcc.client_credentials_token/4","ref":"oidcc.html#client_credentials_token/4"},{"type":"function","doc":"```erlang\n{ok, #oidcc_token{}} =\n  oidcc:client_credentials_token(\n    provider_name,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{scope => [<<\"scope\">>]}\n  ).\n```","title":"Examples - oidcc.client_credentials_token/4","ref":"oidcc.html#client_credentials_token/4-examples"},{"type":"function","doc":"Create Auth Redirect URL","title":"oidcc.create_redirect_url/4","ref":"oidcc.html#create_redirect_url/4"},{"type":"function","doc":"```erlang\n{ok, RedirectUri} =\n      oidcc:create_redirect_url(\n    provider_name,\n    <<\"client_id\">>,\n    <<\"client_secret\">>\n    #{redirect_uri: <<\"https://my.server/return\"}\n  ),\n\n%% RedirectUri = https://my.provider/auth?scope=openid&response_type=code&client_id=client_id&redirect_uri=https%3A%2F%2Fmy.server%2Freturn\n```","title":"Examples - oidcc.create_redirect_url/4","ref":"oidcc.html#create_redirect_url/4-examples"},{"type":"function","doc":"Create Initiate URI for Relaying Party initiated Logout.\n\nSee https://openid.net/specs/openid-connect-rpinitiated-1_0.html#RPLogout.","title":"oidcc.initiate_logout_url/4","ref":"oidcc.html#initiate_logout_url/4"},{"type":"function","doc":"```erlang\n%% Get `Token` from `oidcc_token`\n\n{ok, RedirectUri} =\n  oidcc:initiate_logout_url(\n    Token,\n    provider_name,\n    <<\"client_id\">>,\n    #{post_logout_redirect_uri: <<\"https://my.server/return\"}}\n  ).\n\n%% RedirectUri = https://my.provider/logout?id_token_hint=IDToken&client_id=ClientId&post_logout_redirect_uri=https%3A%2F%2Fmy.server%2Freturn\n```","title":"Examples - oidcc.initiate_logout_url/4","ref":"oidcc.html#initiate_logout_url/4-examples"},{"type":"function","doc":"Introspect the given access token.","title":"oidcc.introspect_token/5","ref":"oidcc.html#introspect_token/5"},{"type":"function","doc":"```erlang\n%% Get AccessToken\n\n{ok, #oidcc_token_introspection{active = True}} =\n  oidcc:introspect_token(\n    AccessToken,\n    provider_name,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{}\n  ).\n```","title":"Examples - oidcc.introspect_token/5","ref":"oidcc.html#introspect_token/5-examples"},{"type":"function","doc":"Retrieve JSON Web Token (JWT) Profile Token.\n\nSee https://datatracker.ietf.org/doc/html/rfc7523#section-4.","title":"oidcc.jwt_profile_token/6","ref":"oidcc.html#jwt_profile_token/6"},{"type":"function","doc":"```erlang\n{ok, KeyJson} = file:read_file(\"jwt-profile.json\"),\nKeyMap = jose:decode(KeyJson),\nKey = jose_jwk:from_pem(maps:get(<<\"key\">>, KeyMap)),\n\n{ok, #oidcc_token{}} =\n  oidcc_token:jwt_profile(\n    <<\"subject\">>,\n    provider_name,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    Key,\n    #{\n     scope => [<<\"scope\">>],\n     kid => maps:get(<<\"keyId\">>, KeyMap)\n    }\n  ).\n```","title":"Examples - oidcc.jwt_profile_token/6","ref":"oidcc.html#jwt_profile_token/6-examples"},{"type":"function","doc":"Refresh Token.","title":"oidcc.refresh_token/5","ref":"oidcc.html#refresh_token/5"},{"type":"function","doc":"```erlang\n%% Get Token and wait for its expiry\n\n{ok, #oidcc_token{}} =\n  oidcc:refresh_token(\n    Token,\n    provider_name,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{expected_subject => <<\"sub_from_initial_id_token\">>}\n  ).\n```","title":"Examples - oidcc.refresh_token/5","ref":"oidcc.html#refresh_token/5-examples"},{"type":"function","doc":"Retrieve the token using the authcode received before and directly validate\nthe result.\n\nThe authcode was sent to the local endpoint by the OpenId Connect provider,\nusing redirects.","title":"oidcc.retrieve_token/5","ref":"oidcc.html#retrieve_token/5"},{"type":"function","doc":"```erlang\n%% Get AuthCode from Redirect\n\n{ok, #oidcc_token{}} =\n  oidcc:retrieve_token(\n    AuthCode,\n    provider_name,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{redirect_uri => <<\"https://example.com/callback\">>}\n  ).\n```","title":"Examples - oidcc.retrieve_token/5","ref":"oidcc.html#retrieve_token/5-examples"},{"type":"function","doc":"Load userinfo for the given token.","title":"oidcc.retrieve_userinfo/5","ref":"oidcc.html#retrieve_userinfo/5"},{"type":"function","doc":"```erlang\n%% Get Token\n\n{ok, #{<<\"sub\">> => Sub}} =\n  oidcc:retrieve_userinfo(\n    Token,\n    provider_name,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{}\n  ).\n```","title":"Examples - oidcc.retrieve_userinfo/5","ref":"oidcc.html#retrieve_userinfo/5-examples"},{"type":"module","doc":"Authentication Utilities","title":"oidcc_auth_util","ref":"oidcc_auth_util.html"},{"type":"type","doc":"","title":"oidcc_auth_util.auth_method/0","ref":"oidcc_auth_util.html#t:auth_method/0"},{"type":"type","doc":"","title":"oidcc_auth_util.error/0","ref":"oidcc_auth_util.html#t:error/0"},{"type":"module","doc":"Functions to start an OpenID Connect Authorization","title":"oidcc_authorization","ref":"oidcc_authorization.html"},{"type":"function","doc":"Create Auth Redirect URL.\n\nFor a high level interface using `m:oidcc_provider_configuration_worker`\nsee `oidcc:create_redirect_url/4`.","title":"oidcc_authorization.create_redirect_url/2","ref":"oidcc_authorization.html#create_redirect_url/2"},{"type":"function","doc":"```erlang\n{ok, ClientContext} =\n    oidcc_client_context:from_configuration_worker(provider_name,\n                                                   <<\"client_id\">>,\n                                                   <<\"client_secret\">>),\n\n{ok, RedirectUri} =\n    oidcc_authorization:create_redirect_url(ClientContext,\n                                            #{redirect_uri: <<\"https://my.server/return\">}),\n\n%% RedirectUri = https://my.provider/auth?scope=openid&response_type=code&client_id=client_id&redirect_uri=https%3A%2F%2Fmy.server%2Freturn\n```","title":"Examples - oidcc_authorization.create_redirect_url/2","ref":"oidcc_authorization.html#create_redirect_url/2-examples"},{"type":"type","doc":"","title":"oidcc_authorization.error/0","ref":"oidcc_authorization.html#t:error/0"},{"type":"type","doc":"Configure authorization redirect URL.\n\nSee https://openid.net/specs/openid-connect-core-1_0.html#AuthRequest.","title":"oidcc_authorization.opts/0","ref":"oidcc_authorization.html#t:opts/0"},{"type":"type","doc":"* `scopes` - list of scopes to request (defaults to `[<<\"openid\">>]`)\n* `state` - state to pass to the provider\n* `nonce` - nonce to pass to the provider\n* `purpose` - purpose of the authorization request, see [https://cdn.connectid.com.au/specifications/oauth2-purpose-01.html]\n* `require_purpose` - whether to require a `purpose` value\n* `pkce_verifier` - PKCE verifier (random string), see [https://datatracker.ietf.org/doc/html/rfc7636#section-4.1]\n* `require_pkce` - whether to require PKCE when getting the token\n* `redirect_uri` - redirect target after authorization is completed\n* `url_extension` - add custom query parameters to the authorization URL\n* `response_mode` - response mode to use (defaults to `<<\"query\">>`)","title":"Parameters - oidcc_authorization.opts/0","ref":"oidcc_authorization.html#t:opts/0-parameters"},{"type":"module","doc":"Backoff Handling\n\nBased on [`db_connection`](https://github.com/elixir-ecto/db_connection/blob/8ef1f2ea54922873590b8939f2dad6b031c5b49c/lib/db_connection/backoff.ex#L24)","title":"oidcc_backoff","ref":"oidcc_backoff.html"},{"type":"type","doc":"","title":"oidcc_backoff.max/0","ref":"oidcc_backoff.html#t:max/0"},{"type":"type","doc":"","title":"oidcc_backoff.min/0","ref":"oidcc_backoff.html#t:min/0"},{"type":"opaque","doc":"","title":"oidcc_backoff.state/0","ref":"oidcc_backoff.html#t:state/0"},{"type":"type","doc":"","title":"oidcc_backoff.type/0","ref":"oidcc_backoff.html#t:type/0"},{"type":"module","doc":"Client Configuration for authorization, token exchange, and userinfo.\n\nFor most projects, it makes sense to use `m:oidcc_provider_configuration_worker` and the high-level\ninterface of `oidcc`. In that case, direct usage of this module is not needed.\n\nTo use the record, import the definition:\n\n```erlang\n-include_lib([\"oidcc/include/oidcc_client_context.hrl\"]).\n```","title":"oidcc_client_context","ref":"oidcc_client_context.html"},{"type":"function","doc":"Apply OpenID Connect / OAuth2 Profiles to the context.\n\nCurrently, the only supported profiles are:\n- `fapi2_security_profile` - https://openid.bitbucket.io/fapi/fapi-2_0-security-profile.html\n- `fapi2_message_signing` - https://openid.bitbucket.io/fapi/fapi-2_0-message-signing.html\n\nIt returns an updated `t:t/0` record and a map of options to\nbe merged into the `m:oidcc_authorization` and `m:oidcc_token` functions.","title":"oidcc_client_context.apply_profiles/2","ref":"oidcc_client_context.html#apply_profiles/2"},{"type":"function","doc":"```erlang\nClientContext = #oidcc_client_context{} = oidcc_client_context:from_...(...),\n\n{#oidcc_client_context{} = ClientContext1, Opts} = oidcc_client_context:apply_profiles(\n  ClientContext,\n  #{\n    profiles => [fapi2_message_signing]\n  }),\n\n{ok, Uri} = oidcc_authorization:create_redirect_uri(\n  ClientContext1,\n  maps:merge(Opts, #{...})\n).\n```","title":"Examples - oidcc_client_context.apply_profiles/2","ref":"oidcc_client_context.html#apply_profiles/2-examples"},{"type":"type","doc":"","title":"oidcc_client_context.authenticated_opts/0","ref":"oidcc_client_context.html#t:authenticated_opts/0"},{"type":"type","doc":"","title":"oidcc_client_context.authenticated_t/0","ref":"oidcc_client_context.html#t:authenticated_t/0"},{"type":"type","doc":"","title":"oidcc_client_context.error/0","ref":"oidcc_client_context.html#t:error/0"},{"type":"function","doc":"Create Client Context from a `m:oidcc_provider_configuration_worker`.\n\nSee `from_configuration_worker/4`.","title":"oidcc_client_context.from_configuration_worker/3","ref":"oidcc_client_context.html#from_configuration_worker/3"},{"type":"function","doc":"Create Client Context from a `m:oidcc_provider_configuration_worker`.","title":"oidcc_client_context.from_configuration_worker/4","ref":"oidcc_client_context.html#from_configuration_worker/4"},{"type":"function","doc":"```erlang\n{ok, Pid} =\n  oidcc_provider_configuration_worker:start_link(#{\n    issuer => <<\"https://login.salesforce.com\">>\n  }),\n\n{ok, #oidcc_client_context{}} =\n  oidcc_client_context:from_configuration_worker(Pid,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>).\n```\n\n```erlang\n{ok, Pid} =\n  oidcc_provider_configuration_worker:start_link(#{\n    issuer => <<\"https://login.salesforce.com\">>,\n    name => {local, salesforce_provider}\n  }),\n\n{ok, #oidcc_client_context{}} =\n  oidcc_client_context:from_configuration_worker(\n    salesforce_provider,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{client_jwks => jose_jwk:generate_key(16)}\n  ).\n```","title":"Examples - oidcc_client_context.from_configuration_worker/4","ref":"oidcc_client_context.html#from_configuration_worker/4-examples"},{"type":"function","doc":"Create Client Context manually.\n\nSee `from_manual/5`.","title":"oidcc_client_context.from_manual/4","ref":"oidcc_client_context.html#from_manual/4"},{"type":"function","doc":"Create Client Context manually.","title":"oidcc_client_context.from_manual/5","ref":"oidcc_client_context.html#from_manual/5"},{"type":"function","doc":"```erlang\n{ok, Configuration} =\n  oidcc_provider_configuration:load_configuration(<<\"https://login.salesforce.com\">>, []),\n\n#oidcc_provider_configuration{jwks_uri = JwksUri} = Configuration,\n\n{ok, Jwks} = oidcc_provider_configuration:load_jwks(JwksUri, []).\n\n#oidcc_client_context{} =\n  oidcc_client_context:from_manual(\n    Metadata,\n    Jwks,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{client_jwks => jose_jwk:generate_key(16)}\n  ).\n```","title":"Examples - oidcc_client_context.from_manual/5","ref":"oidcc_client_context.html#from_manual/5-examples"},{"type":"type","doc":"","title":"oidcc_client_context.opts/0","ref":"oidcc_client_context.html#t:opts/0"},{"type":"type","doc":"","title":"oidcc_client_context.t/0","ref":"oidcc_client_context.html#t:t/0"},{"type":"type","doc":"","title":"oidcc_client_context.unauthenticated_opts/0","ref":"oidcc_client_context.html#t:unauthenticated_opts/0"},{"type":"type","doc":"","title":"oidcc_client_context.unauthenticated_t/0","ref":"oidcc_client_context.html#t:unauthenticated_t/0"},{"type":"module","doc":"Dynamic Client Registration Utilities.\n\nSee https://openid.net/specs/openid-connect-registration-1_0.html#ClientMetadata.","title":"oidcc_client_registration","ref":"oidcc_client_registration.html"},{"type":"module","doc":"To use the record, import the definition:\n\n```erlang\n-include_lib([\"oidcc/include/oidcc_client_registration.hrl\"]).\n```","title":"Records - oidcc_client_registration","ref":"oidcc_client_registration.html#module-records"},{"type":"module","doc":"See [`Oidcc.ClientRegistration`](`m:'Elixir.Oidcc.ClientRegistration'`).","title":"Telemetry - oidcc_client_registration","ref":"oidcc_client_registration.html#module-telemetry"},{"type":"type","doc":"","title":"oidcc_client_registration.error/0","ref":"oidcc_client_registration.html#t:error/0"},{"type":"type","doc":"Configure configuration loading / parsing.","title":"oidcc_client_registration.opts/0","ref":"oidcc_client_registration.html#t:opts/0"},{"type":"type","doc":"* `initial_access_token` - Access Token for registration\n* `request_opts` - config for HTTP request","title":"Parameters - oidcc_client_registration.opts/0","ref":"oidcc_client_registration.html#t:opts/0-parameters"},{"type":"function","doc":"Register Client.","title":"oidcc_client_registration.register/3","ref":"oidcc_client_registration.html#register/3"},{"type":"function","doc":"```erlang\n{ok, ProviderConfiguration} =\n  oidcc_provider_configuration:load_configuration(\"https://your.issuer\"),\n\n{ok, #oidcc_client_registration_response{\n  client_id = ClientId,\n  client_secret = ClientSecret\n}} =\n  oidcc_client_registration:register(\n    ProviderConfiguration,\n    #oidcc_client_registration{\n      redirect_uris = [\"https://your.application.com/oidcc/callback\"]\n    },\n    #{initial_access_token => <<\"optional token you got from the provider\">>}\n  ).\n```","title":"Examples - oidcc_client_registration.register/3","ref":"oidcc_client_registration.html#register/3-examples"},{"type":"type","doc":"Record containing Client Registration Response.\n\nSee https://openid.net/specs/openid-connect-registration-1_0.html#RegistrationResponse.\n\nAll unrecognized fields are stored in `extra_fields`.","title":"oidcc_client_registration.response/0","ref":"oidcc_client_registration.html#t:response/0"},{"type":"type","doc":"Record containing Client Registration Metadata.\n\nSee https://openid.net/specs/openid-connect-registration-1_0.html#ClientMetadata and\nhttps://openid.net/specs/openid-connect-rpinitiated-1_0.html#ClientMetadata.\n\nAll unrecognized fields are stored in `extra_fields`.","title":"oidcc_client_registration.t/0","ref":"oidcc_client_registration.html#t:t/0"},{"type":"module","doc":"Response Decoding Utils","title":"oidcc_decode_util","ref":"oidcc_decode_util.html"},{"type":"type","doc":"","title":"oidcc_decode_util.error/0","ref":"oidcc_decode_util.html#t:error/0"},{"type":"module","doc":"HTTP Client Utilities","title":"oidcc_http_util","ref":"oidcc_http_util.html"},{"type":"type","doc":"","title":"oidcc_http_util.error/0","ref":"oidcc_http_util.html#t:error/0"},{"type":"function","doc":"","title":"oidcc_http_util.headers_to_cache_deadline/2","ref":"oidcc_http_util.html#headers_to_cache_deadline/2"},{"type":"type","doc":"See `httpc:request/5`.","title":"oidcc_http_util.http_header/0","ref":"oidcc_http_util.html#t:http_header/0"},{"type":"type","doc":"See `httpc:request/5` for additional errors.","title":"oidcc_http_util.httpc_error/0","ref":"oidcc_http_util.html#t:httpc_error/0"},{"type":"type","doc":"See `uri_string:compose_query/1`.","title":"oidcc_http_util.query_params/0","ref":"oidcc_http_util.html#t:query_params/0"},{"type":"type","doc":"See `httpc:request/5`.","title":"oidcc_http_util.request_opts/0","ref":"oidcc_http_util.html#t:request_opts/0"},{"type":"type","doc":"* `timeout` - timeout for request\n* `ssl` - TLS config","title":"Parameters - oidcc_http_util.request_opts/0","ref":"oidcc_http_util.html#t:request_opts/0-parameters"},{"type":"type","doc":"","title":"oidcc_http_util.telemetry_opts/0","ref":"oidcc_http_util.html#t:telemetry_opts/0"},{"type":"module","doc":"JWT Utilities","title":"oidcc_jwt_util","ref":"oidcc_jwt_util.html"},{"type":"type","doc":"","title":"oidcc_jwt_util.claims/0","ref":"oidcc_jwt_util.html#t:claims/0"},{"type":"type","doc":"","title":"oidcc_jwt_util.error/0","ref":"oidcc_jwt_util.html#t:error/0"},{"type":"type","doc":"","title":"oidcc_jwt_util.refresh_jwks_for_unknown_kid_fun/0","ref":"oidcc_jwt_util.html#t:refresh_jwks_for_unknown_kid_fun/0"},{"type":"module","doc":"Logout from the OpenID Provider.","title":"oidcc_logout","ref":"oidcc_logout.html"},{"type":"type","doc":"","title":"oidcc_logout.error/0","ref":"oidcc_logout.html#t:error/0"},{"type":"function","doc":"Initiate URI for Relaying Party initiated Logout.\n\nSee https://openid.net/specs/openid-connect-rpinitiated-1_0.html#RPLogout.\n\nFor a high level interface using `m:oidcc_provider_configuration_worker`\nsee `oidcc:initiate_logout_url/4`.","title":"oidcc_logout.initiate_url/3","ref":"oidcc_logout.html#initiate_url/3"},{"type":"function","doc":"```erlang\n{ok, ClientContext} = oidcc_client_context:from_configuration_worker(\n  provider_name,\n  <<\"client_id\">>,\n  unauthenticated\n),\n\n%% Get `Token` from `oidcc_token`\n\n{ok, RedirectUri} =\n  oidcc_logout:initiate_url(\n    Token,\n    ClientContext,\n    #{post_logout_redirect_uri: <<\"https://my.server/return\">}\n),\n\n%% RedirectUri = https://my.provider/logout?id_token_hint=IDToken&client_id=ClientId&post_logout_redirect_uri=https%3A%2F%2Fmy.server%2Freturn\n```","title":"Examples - oidcc_logout.initiate_url/3","ref":"oidcc_logout.html#initiate_url/3-examples"},{"type":"type","doc":"Configure Relaying Party initiated Logout URI.\n\nSee https://openid.net/specs/openid-connect-rpinitiated-1_0.html#RPLogout.","title":"oidcc_logout.initiate_url_opts/0","ref":"oidcc_logout.html#t:initiate_url_opts/0"},{"type":"type","doc":"* `logout_hint` - logout_hint to pass to the provider\n* `post_logout_redirect_uri` - Post Logout Redirect URI to pass to the provider\n* `state` - state to pass to the provider\n* `ui_locales` - UI locales to pass to the provider\n* `extra_query_params` - extra query params to add to the URI","title":"Parameters - oidcc_logout.initiate_url_opts/0","ref":"oidcc_logout.html#t:initiate_url_opts/0-parameters"},{"type":"module","doc":"OpenID Profile Utilities","title":"oidcc_profile","ref":"oidcc_profile.html"},{"type":"type","doc":"","title":"oidcc_profile.error/0","ref":"oidcc_profile.html#t:error/0"},{"type":"type","doc":"","title":"oidcc_profile.opts/0","ref":"oidcc_profile.html#t:opts/0"},{"type":"type","doc":"","title":"oidcc_profile.opts_no_profiles/0","ref":"oidcc_profile.html#t:opts_no_profiles/0"},{"type":"type","doc":"","title":"oidcc_profile.profile/0","ref":"oidcc_profile.html#t:profile/0"},{"type":"module","doc":"Tooling to load and parse Openid Configuration.","title":"oidcc_provider_configuration","ref":"oidcc_provider_configuration.html"},{"type":"module","doc":"To use the record, import the definition:\n\n```erlang\n-include_lib([\"oidcc/include/oidcc_provider_configuration.hrl\"]).\n```","title":"Records - oidcc_provider_configuration","ref":"oidcc_provider_configuration.html#module-records"},{"type":"module","doc":"See [`Oidcc.ProviderConfiguration`](`m:'Elixir.Oidcc.ProviderConfiguration'`).","title":"Telemetry - oidcc_provider_configuration","ref":"oidcc_provider_configuration.html#module-telemetry"},{"type":"function","doc":"See `decode_configuration/2`.","title":"oidcc_provider_configuration.decode_configuration/1","ref":"oidcc_provider_configuration.html#decode_configuration/1"},{"type":"function","doc":"Decode JSON into a `t:oidcc_provider_configuration:t/0` record.","title":"oidcc_provider_configuration.decode_configuration/2","ref":"oidcc_provider_configuration.html#decode_configuration/2"},{"type":"function","doc":"```erlang\n{ok, {{\"HTTP/1.1\",200,\"OK\"}, _Headers, Body}} =\n  httpc:request(\"https://accounts.google.com/.well-known/openid-configuration\"),\n\n{ok, DecodedJson} = your_json_lib:decode(Body),\n\n{ok, #oidcc_provider_configuration{}} =\n  oidcc_provider_configuration:decode_configuration(DecodedJson).\n```","title":"Examples - oidcc_provider_configuration.decode_configuration/2","ref":"oidcc_provider_configuration.html#decode_configuration/2-examples"},{"type":"type","doc":"","title":"oidcc_provider_configuration.error/0","ref":"oidcc_provider_configuration.html#t:error/0"},{"type":"function","doc":"See `load_configuration/2`.","title":"oidcc_provider_configuration.load_configuration/1","ref":"oidcc_provider_configuration.html#load_configuration/1"},{"type":"function","doc":"Load OpenID Configuration into a `t:oidcc_provider_configuration:t/0` record.","title":"oidcc_provider_configuration.load_configuration/2","ref":"oidcc_provider_configuration.html#load_configuration/2"},{"type":"function","doc":"```erlang\n{ok, #oidcc_provider_configuration{}} =\n  oidcc_provider_configuration:load_configuration(\"https://accounts.google.com\").\n```","title":"Examples - oidcc_provider_configuration.load_configuration/2","ref":"oidcc_provider_configuration.html#load_configuration/2-examples"},{"type":"function","doc":"Load JWKs into a `t:jose_jwk:key/0` record.","title":"oidcc_provider_configuration.load_jwks/2","ref":"oidcc_provider_configuration.html#load_jwks/2"},{"type":"function","doc":"```erlang\n{ok, #jose_jwk{}} =\n  oidcc_provider_configuration:load_jwks(\"https://www.googleapis.com/oauth2/v3/certs\").\n```","title":"Examples - oidcc_provider_configuration.load_jwks/2","ref":"oidcc_provider_configuration.html#load_jwks/2-examples"},{"type":"type","doc":"Configure configuration loading / parsing.","title":"oidcc_provider_configuration.opts/0","ref":"oidcc_provider_configuration.html#t:opts/0"},{"type":"type","doc":"* `fallback_expiry` - How long to keep configuration cached if the server doesn't specify expiry.\n* `request_opts` - config for HTTP request.","title":"Parameters - oidcc_provider_configuration.opts/0","ref":"oidcc_provider_configuration.html#t:opts/0-parameters"},{"type":"type","doc":"Allow Specification Non-compliance.","title":"oidcc_provider_configuration.quirks/0","ref":"oidcc_provider_configuration.html#t:quirks/0"},{"type":"type","doc":"* `allow_unsafe_http` - Allow unsafe HTTP. Use this for development\n  providers and **never in production**.\n* `document_overrides` - a map to merge with the real OIDD document,\n  in case the OP left out some values.\n* `issuer_regex` - Optional regex pattern to match against the issuer claim\n  instead of requiring an exact match. This may be necessary for certain providers that do not\n  conform to the OpenID specification, such as Microsoft Entra ID where\n  the issuer is 'https://login.microsoftonline.com/{tenantid}/v2.0' in the\n  [OpenID configuration](https://login.microsoftonline.com/common/v2.0/.well-known/openid-configuration).","title":"Exceptions - oidcc_provider_configuration.quirks/0","ref":"oidcc_provider_configuration.html#t:quirks/0-exceptions"},{"type":"type","doc":"Record containing OpenID and OAuth 2.0 Configuration.\n\nSee:\n* https://openid.net/specs/openid-connect-discovery-1_0.html#ProviderMetadata\n* https://datatracker.ietf.org/doc/html/draft-jones-oauth-discovery-01#section-4.1\n* https://openid.net/specs/openid-connect-rpinitiated-1_0.html#OPMetadata\n\nAll unrecognized fields are stored in `extra_fields`.","title":"oidcc_provider_configuration.t/0","ref":"oidcc_provider_configuration.html#t:t/0"},{"type":"module","doc":"OIDC Config Provider Worker\n\nLoads and continuously refreshes the OIDC configuration and JWKs.\n\nThe worker supports reading values concurrently via an ETS table. To use\nthis performance improvement, the worker has to be registered with a\n`{local, Name}`. No name / `{global, Name}` and `{via, RegModule, ViaName}`\nare not supported.","title":"oidcc_provider_configuration_worker","ref":"oidcc_provider_configuration_worker.html"},{"type":"function","doc":"Get Parsed Jwks.","title":"oidcc_provider_configuration_worker.get_jwks/1","ref":"oidcc_provider_configuration_worker.html#get_jwks/1"},{"type":"function","doc":"Get Configuration.","title":"oidcc_provider_configuration_worker.get_provider_configuration/1","ref":"oidcc_provider_configuration_worker.html#get_provider_configuration/1"},{"type":"type","doc":"Configuration Options\n\n* `name` - The gen_server name of the provider.\n* `issuer` - The issuer URI.\n* `provider_configuration_opts` - Options for the provider configuration\n  fetching.\n* `backoff_min` - The minimum backoff interval in ms (default: `1_000`).\n* `backoff_max` - The maximum backoff interval in ms (default: `30_000`).\n* `backoff_type` - The backoff strategy, `stop` for no backoff and to stop,\n  `exponential` for exponential, `random` for random, and `random_exponential`\n  for random exponential (default: `stop`).","title":"oidcc_provider_configuration_worker.opts/0","ref":"oidcc_provider_configuration_worker.html#t:opts/0"},{"type":"function","doc":"Refresh Configuration.","title":"oidcc_provider_configuration_worker.refresh_configuration/1","ref":"oidcc_provider_configuration_worker.html#refresh_configuration/1"},{"type":"function","doc":"```erlang\n{ok, Pid} =\n  oidcc_provider_configuration_worker:start_link(#{\n    issuer => <<\"https://accounts.google.com\">>\n  }).\n\n%% Later\n\noidcc_provider_configuration_worker:refresh_configuration(Pid).\n```","title":"Examples - oidcc_provider_configuration_worker.refresh_configuration/1","ref":"oidcc_provider_configuration_worker.html#refresh_configuration/1-examples"},{"type":"function","doc":"Refresh JWKs.","title":"oidcc_provider_configuration_worker.refresh_jwks/1","ref":"oidcc_provider_configuration_worker.html#refresh_jwks/1"},{"type":"function","doc":"```erlang\n{ok, Pid} =\n  oidcc_provider_configuration_worker:start_link(#{\n    issuer => <<\"https://accounts.google.com\">>\n  }).\n\n%% Later\n\noidcc_provider_configuration_worker:refresh_jwks(Pid).\n```","title":"Examples - oidcc_provider_configuration_worker.refresh_jwks/1","ref":"oidcc_provider_configuration_worker.html#refresh_jwks/1-examples"},{"type":"function","doc":"Refresh JWKs if the provided `Kid` is not matching any currently loaded keys.","title":"oidcc_provider_configuration_worker.refresh_jwks_for_unknown_kid/2","ref":"oidcc_provider_configuration_worker.html#refresh_jwks_for_unknown_kid/2"},{"type":"function","doc":"```erlang\n{ok, Pid} =\n  oidcc_provider_configuration_worker:start_link(#{\n    issuer => <<\"https://accounts.google.com\">>\n  }).\n\noidcc_provider_configuration_worker:refresh_jwks_for_unknown_kid(Pid, <<\"kid\">>).\n```","title":"Examples - oidcc_provider_configuration_worker.refresh_jwks_for_unknown_kid/2","ref":"oidcc_provider_configuration_worker.html#refresh_jwks_for_unknown_kid/2-examples"},{"type":"function","doc":"Start Configuration Provider.","title":"oidcc_provider_configuration_worker.start_link/1","ref":"oidcc_provider_configuration_worker.html#start_link/1"},{"type":"function","doc":"```erlang\n{ok, Pid} =\n  oidcc_provider_configuration_worker:start_link(#{\n    issuer => <<\"https://accounts.google.com\">>,\n    name => {local, google_config_provider}\n  }).\n```\n\n```erlang\n%% ...\n-behaviour(supervisor).\n\n%% ...\n\ninit(_opts) ->\n  SupFlags = #{strategy => one_for_one, intensity => 1, period => 5},\n  ChildSpecs = [#{id => google_config_provider,\n    start => {oidcc_provider_configuration_worker,\n              start_link,\n              [\n                #{issuer => <<\"https://accounts.google.com\">>}\n              ]},\n    restart => permanent,\n    type => worker,\n    modules => [oidcc_provider_configuration_worker]}],\n  {ok, {SupFlags, ChildSpecs}}.\n```","title":"Examples - oidcc_provider_configuration_worker.start_link/1","ref":"oidcc_provider_configuration_worker.html#start_link/1-examples"},{"type":"module","doc":"OpenID Scope Utilities","title":"oidcc_scope","ref":"oidcc_scope.html"},{"type":"function","doc":"Parse `t:t/0` into `t:scopes/0`.","title":"oidcc_scope.parse/1","ref":"oidcc_scope.html#parse/1"},{"type":"function","doc":"```erlang\n[<<\"openid\">>, <<\"profile\">>] = oidcc_scope:parse(<<\"openid profile\">>).\n```","title":"Examples - oidcc_scope.parse/1","ref":"oidcc_scope.html#parse/1-examples"},{"type":"type","doc":"","title":"oidcc_scope.scopes/0","ref":"oidcc_scope.html#t:scopes/0"},{"type":"function","doc":"Compose `t:scopes/0` into `t:t/0`.","title":"oidcc_scope.scopes_to_bin/1","ref":"oidcc_scope.html#scopes_to_bin/1"},{"type":"function","doc":"```erlang\n<<\"openid profile email\">> = oidcc_scope:scopes_to_bin(\n  [<<\"openid\">>, profile, \"email\"]).\n```","title":"Examples - oidcc_scope.scopes_to_bin/1","ref":"oidcc_scope.html#scopes_to_bin/1-examples"},{"type":"type","doc":"","title":"oidcc_scope.t/0","ref":"oidcc_scope.html#t:t/0"},{"type":"module","doc":"Facilitate OpenID Code/Token Exchanges.","title":"oidcc_token","ref":"oidcc_token.html"},{"type":"module","doc":"To use the records, import the definition:\n\n```erlang\n-include_lib([\"oidcc/include/oidcc_token.hrl\"]).\n```","title":"Records - oidcc_token","ref":"oidcc_token.html#module-records"},{"type":"module","doc":"See [`Oidcc.Token`](`m:'Elixir.Oidcc.Token'`).","title":"Telemetry - oidcc_token","ref":"oidcc_token.html#module-telemetry"},{"type":"type","doc":"Access Token Wrapper.","title":"oidcc_token.access/0","ref":"oidcc_token.html#t:access/0"},{"type":"type","doc":"* `token` - The retrieved token.\n* `expires` - Number of seconds the token is valid.","title":"Fields - oidcc_token.access/0","ref":"oidcc_token.html#t:access/0-fields"},{"type":"function","doc":"Authorization headers\n\nGenerate a map of authorization headers to use when using the given\naccess token to access an API endpoint.","title":"oidcc_token.authorization_headers/4","ref":"oidcc_token.html#authorization_headers/4"},{"type":"function","doc":"```erlang\n{ok, ClientContext} =\n    oidcc_client_context:from_configuration_worker(provider_name,\n                                                    <<\"client_id\">>,\n                                                    <<\"client_secret\">>),\n%% Get Access Token record from somewhere\nHeaders =\n    oidcc:authorization_headers(AccessTokenRecord, :get, Url, ClientContext).\n```","title":"Examples - oidcc_token.authorization_headers/4","ref":"oidcc_token.html#authorization_headers/4-examples"},{"type":"function","doc":"","title":"oidcc_token.authorization_headers/5","ref":"oidcc_token.html#authorization_headers/5"},{"type":"type","doc":"","title":"oidcc_token.authorization_headers_opts/0","ref":"oidcc_token.html#t:authorization_headers_opts/0"},{"type":"function","doc":"Retrieve Client Credential Token\n\nSee https://datatracker.ietf.org/doc/html/rfc6749#section-1.3.4\n\nFor a high level interface using `m:oidcc_provider_configuration_worker`\nsee `oidcc:client_credentials_token/4`.","title":"oidcc_token.client_credentials/2","ref":"oidcc_token.html#client_credentials/2"},{"type":"function","doc":"```erlang\n{ok, ClientContext} =\n  oidcc_client_context:from_configuration_worker(provider_name,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>),\n\n{ok, #oidcc_token{}} =\n  oidcc_token:client_credentials(ClientContext,\n                                 #{scope => [<<\"scope\">>]}).\n```","title":"Examples - oidcc_token.client_credentials/2","ref":"oidcc_token.html#client_credentials/2-examples"},{"type":"type","doc":"","title":"oidcc_token.client_credentials_opts/0","ref":"oidcc_token.html#t:client_credentials_opts/0"},{"type":"type","doc":"","title":"oidcc_token.error/0","ref":"oidcc_token.html#t:error/0"},{"type":"type","doc":"ID Token Wrapper.","title":"oidcc_token.id/0","ref":"oidcc_token.html#t:id/0"},{"type":"type","doc":"* `token` - The retrieved token.\n* `claims` - Unpacked claims of the verified token.","title":"Fields - oidcc_token.id/0","ref":"oidcc_token.html#t:id/0-fields"},{"type":"function","doc":"Retrieve JSON Web Token (JWT) Profile Token\n\nSee [https://datatracker.ietf.org/doc/html/rfc7523#section-4]\n\nFor a high level interface using {@link oidcc_provider_configuration_worker}\nsee {@link oidcc:jwt_profile_token/6}.","title":"oidcc_token.jwt_profile/4","ref":"oidcc_token.html#jwt_profile/4"},{"type":"function","doc":"```erlang\n{ok, ClientContext} =\n  oidcc_client_context:from_configuration_worker(provider_name,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>),\n\n{ok, KeyJson} = file:read_file(\"jwt-profile.json\"),\nKeyMap = jose:decode(KeyJson),\nKey = jose_jwk:from_pem(maps:get(<<\"key\">>, KeyMap)),\n\n{ok, #oidcc_token{}} =\n  oidcc_token:jwt_profile(<<\"subject\">>,\n                          ClientContext,\n                          Key,\n                          #{scope => [<<\"scope\">>],\n                            kid => maps:get(<<\"keyId\">>, KeyMap)}).\n```","title":"Examples - oidcc_token.jwt_profile/4","ref":"oidcc_token.html#jwt_profile/4-examples"},{"type":"type","doc":"","title":"oidcc_token.jwt_profile_opts/0","ref":"oidcc_token.html#t:jwt_profile_opts/0"},{"type":"type","doc":"Refresh Token Wrapper.","title":"oidcc_token.refresh/0","ref":"oidcc_token.html#t:refresh/0"},{"type":"type","doc":"* `token` - The retrieved token.","title":"Fields - oidcc_token.refresh/0","ref":"oidcc_token.html#t:refresh/0-fields"},{"type":"function","doc":"Refresh Token\n\nFor a high level interface using `m:oidcc_provider_configuration_worker`\nsee `oidcc:refresh_token/5`.","title":"oidcc_token.refresh/3","ref":"oidcc_token.html#refresh/3"},{"type":"function","doc":"```erlang\n{ok, ClientContext} =\n  oidcc_client_context:from_configuration_worker(provider_name,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>),\n\n%% Get AuthCode from Redirect\n\n{ok, Token} =\n  oidcc_token:retrieve(AuthCode, ClientContext, #{\n    redirect_uri => <<\"https://example.com/callback\">>}).\n\n%% Later\n\n{ok, #oidcc_token{}} =\n  oidcc_token:refresh(Token,\n                      ClientContext,\n                      #{expected_subject => <<\"sub_from_initial_id_token\">>}).\n```","title":"Examples - oidcc_token.refresh/3","ref":"oidcc_token.html#refresh/3-examples"},{"type":"type","doc":"","title":"oidcc_token.refresh_opts/0","ref":"oidcc_token.html#t:refresh_opts/0"},{"type":"type","doc":"See `t:refresh_opts_no_sub/0`.","title":"oidcc_token.refresh_opts_no_sub/0","ref":"oidcc_token.html#t:refresh_opts_no_sub/0"},{"type":"function","doc":"Retrieve the token using the authcode received before and directly validate\nthe result.\n\nThe authcode was sent to the local endpoint by the OpenId Connect provider,\nusing redirects.\n\nFor a high level interface using `m:oidcc_provider_configuration_worker`\nsee `oidcc:retrieve_token/5`.","title":"oidcc_token.retrieve/3","ref":"oidcc_token.html#retrieve/3"},{"type":"function","doc":"```erlang\n{ok, ClientContext} =\n  oidcc_client_context:from_configuration_worker(provider_name,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>),\n\n%% Get AuthCode from Redirect\n\n{ok, #oidcc_token{}} =\n  oidcc:retrieve(AuthCode, ClientContext, #{\n    redirect_uri => <<\"https://example.com/callback\">>}).\n```","title":"Examples - oidcc_token.retrieve/3","ref":"oidcc_token.html#retrieve/3-examples"},{"type":"type","doc":"Options for retrieving a token.\n\nSee https://datatracker.ietf.org/doc/html/rfc6749#section-4.1.3.","title":"oidcc_token.retrieve_opts/0","ref":"oidcc_token.html#t:retrieve_opts/0"},{"type":"type","doc":"* `pkce_verifier` - PKCE verifier (random string previously given to\n  `m:oidcc_authorization`), see\n  https://datatracker.ietf.org/doc/html/rfc7636#section-4.1.\n* `require_pkce` - whether to require PKCE when getting the token.\n* `nonce` - Nonce to check.\n* `scope` - Scope to store with the token.\n* `refresh_jwks` - How to handle tokens with an unknown `kid`.\n  See `t:oidcc_jwt_util:refresh_jwks_for_unknown_kid_fun/0`.\n* `redirect_uri` - Redirect URI given to `oidcc_authorization:create_redirect_url/2`.\n* `dpop_nonce` - if using DPoP, the `nonce` value to use in the proof claim.\n* `trusted_audiences` - if present, a list of additional audience values to\n  accept. Defaults to `any` which allows any additional values.\n* `token_request_claims` - Additional claims to use with the token request.","title":"Fields - oidcc_token.retrieve_opts/0","ref":"oidcc_token.html#t:retrieve_opts/0-fields"},{"type":"type","doc":"Token Response Wrapper.","title":"oidcc_token.t/0","ref":"oidcc_token.html#t:t/0"},{"type":"type","doc":"* `id` - `t:id/0`.\n* `access` - `t:access/0`.\n* `refresh` - `t:refresh/0`.\n* `scope` - `t:oidcc_scope:scopes/0`.","title":"Fields - oidcc_token.t/0","ref":"oidcc_token.html#t:t/0-fields"},{"type":"function","doc":"Validate ID Token\n\nUsually the id token is validated using `retrieve/3`.\n\nIf you get the token passed from somewhere else, this function can validate it.","title":"oidcc_token.validate_id_token/3","ref":"oidcc_token.html#validate_id_token/3"},{"type":"function","doc":"* `iss` claim must match the issuer of the provider, or match the regex pattern if `issuer_regex` is configured in quirks.\n* `nonce` claim must match the `nonce` option.\n* `aud` claim must match the `trusted_audiences` option.\n* `azp` claim must match the client id.\n* `exp` claim must be in the future.\n* `nbf` claim must be in the past.","title":"Validations - oidcc_token.validate_id_token/3","ref":"oidcc_token.html#validate_id_token/3-validations"},{"type":"function","doc":"```erlang\n{ok, ClientContext} =\n  oidcc_client_context:from_configuration_worker(provider_name,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>),\n\n%% Get IdToken from somewhere\n\n{ok, Claims} =\n  oidcc:validate_id_token(IdToken, ClientContext, ExpectedNonce).\n```","title":"Examples - oidcc_token.validate_id_token/3","ref":"oidcc_token.html#validate_id_token/3-examples"},{"type":"function","doc":"You can use a regex pattern to validate the issuer claim by adding an `issuer_regex`\nto the quirks map when creating the provider configuration. See the documentation for `validate_jwt/3`\nfor more details.","title":"Regex Issuer Validation - oidcc_token.validate_id_token/3","ref":"oidcc_token.html#validate_id_token/3-regex-issuer-validation"},{"type":"function","doc":"Validate the JARM response, returning the valid claims as a map.\n\nThe response was sent to the local endpoint by the OpenId Connect provider,\nusing redirects.","title":"oidcc_token.validate_jarm/3","ref":"oidcc_token.html#validate_jarm/3"},{"type":"function","doc":"```erlang\n{ok, ClientContext} =\n  oidcc_client_context:from_configuration_worker(provider_name,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>),\n\n%% Get Response from Redirect\n\n{ok, #{<<\"code\">> := AuthCode}} =\n  oidcc:validate_jarm(Response, ClientContext, #{}),\n\n{ok, #oidcc_token{}} = oidcc:retrieve(AuthCode, ClientContext,\n  #{redirect_uri => <<\"https://redirect.example/\">>}).\n```","title":"Examples - oidcc_token.validate_jarm/3","ref":"oidcc_token.html#validate_jarm/3-examples"},{"type":"type","doc":"Options for refreshing a token.\n\nSee https://datatracker.ietf.org/doc/html/rfc6749#section-4.1.3.","title":"oidcc_token.validate_jarm_opts/0","ref":"oidcc_token.html#t:validate_jarm_opts/0"},{"type":"type","doc":"* `scope` - Scope to store with the token.\n* `refresh_jwks` - How to handle tokens with an unknown `kid`.\n  See `t:oidcc_jwt_util:refresh_jwks_for_unknown_kid_fun/0`.\n* `expected_subject` - `sub` of the original token.","title":"Fields - oidcc_token.validate_jarm_opts/0","ref":"oidcc_token.html#t:validate_jarm_opts/0-fields"},{"type":"function","doc":"Validate JWT\n\nValidates a generic JWT (such as an access token) from the given provider.\nUseful if the issuer is shared between multiple applications, and the access token\ngenerated for a user at one client is used to validate their access at another client.\n\nValidating an arbitrary JWT token (not an ID token) is not covered by the OpenID\nConnect specification. Therefore the signing / encryption algorithms are not\nderieved from the provider configuration, but must be provided by the caller.","title":"oidcc_token.validate_jwt/3","ref":"oidcc_token.html#validate_jwt/3"},{"type":"function","doc":"* `iss` claim must match the issuer of the provider, or match the regex pattern if `issuer_regex` is configured in quirks.\n* `aud` claim must match the `trusted_audiences` option.\n* `exp` claim must be in the future.\n* `nbf` claim must be in the past.","title":"Validations - oidcc_token.validate_jwt/3","ref":"oidcc_token.html#validate_jwt/3-validations"},{"type":"function","doc":"```erlang\n{ok, ClientContext} =\n    oidcc_client_context:from_configuration_worker(provider_name,\n                                                <<\"client_id\">>,\n                                                <<\"client_secret\">>),\n%% Get Jwt from Authorization header\nJwt = <<\"jwt\">>,\n\nOpts = #{\n  signing_algs => [<<\"RS256\">>]\n},\n\n{ok, Claims} =\n    oidcc:validate_jwt(Jwt, ClientContext, Opts).\n```","title":"Examples - oidcc_token.validate_jwt/3","ref":"oidcc_token.html#validate_jwt/3-examples"},{"type":"function","doc":"You can use a regex pattern to validate the issuer claim by adding an `issuer_regex`\nto the quirks map when creating the provider configuration:\n\n```erlang\n{ok, {ProviderConfig, _}} =\n    oidcc_provider_configuration:load_configuration(Issuer, #{\n        quirks => #{\n            issuer_regex => <<\"^https://accounts\\\\.example\\\\.com/[a-z0-9]+\">>\n        }\n    }),\n```\n\nThis will allow tokens with issuer claims that match the regex pattern to validate successfully.","title":"Regex Issuer Validation - oidcc_token.validate_jwt/3","ref":"oidcc_token.html#validate_jwt/3-regex-issuer-validation"},{"type":"type","doc":"","title":"oidcc_token.validate_jwt_opts/0","ref":"oidcc_token.html#t:validate_jwt_opts/0"},{"type":"module","doc":"OAuth Token Introspection.\n\nSee https://datatracker.ietf.org/doc/html/rfc7662.","title":"oidcc_token_introspection","ref":"oidcc_token_introspection.html"},{"type":"module","doc":"To use the records, import the definition:\n\n```erlang\n-include_lib([\"oidcc/include/oidcc_token_introspection.hrl\"]).\n```","title":"Records - oidcc_token_introspection","ref":"oidcc_token_introspection.html#module-records"},{"type":"module","doc":"See [`Oidcc.TokenIntrospection`](`m:'Elixir.Oidcc.TokenIntrospection'`).","title":"Telemetry - oidcc_token_introspection","ref":"oidcc_token_introspection.html#module-telemetry"},{"type":"type","doc":"","title":"oidcc_token_introspection.error/0","ref":"oidcc_token_introspection.html#t:error/0"},{"type":"function","doc":"Introspect the given access token.\n\nFor a high level interface using `m:oidcc_provider_configuration_worker`\nsee `oidcc:introspect_token/5`.","title":"oidcc_token_introspection.introspect/3","ref":"oidcc_token_introspection.html#introspect/3"},{"type":"function","doc":"```erlang\n{ok, ClientContext} =\n  oidcc_client_context:from_configuration_worker(provider_name,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>),\n\n%% Get AccessToken\n\n{ok, #oidcc_token_introspection{active = True}} =\n  oidcc_token_introspection:introspect(AccessToken, ClientContext, #{}).\n```","title":"Examples - oidcc_token_introspection.introspect/3","ref":"oidcc_token_introspection.html#introspect/3-examples"},{"type":"type","doc":"","title":"oidcc_token_introspection.opts/0","ref":"oidcc_token_introspection.html#t:opts/0"},{"type":"type","doc":"Introspection Result.\n\nSee https://datatracker.ietf.org/doc/html/rfc7662#section-2.2.","title":"oidcc_token_introspection.t/0","ref":"oidcc_token_introspection.html#t:t/0"},{"type":"module","doc":"OpenID Connect Userinfo\n\nSee https://openid.net/specs/openid-connect-core-1_0.html#UserInfo","title":"oidcc_userinfo","ref":"oidcc_userinfo.html"},{"type":"module","doc":"See [`Oidcc.Userinfo`](`m:'Elixir.Oidcc.Userinfo'`).","title":"Telemetry - oidcc_userinfo","ref":"oidcc_userinfo.html#module-telemetry"},{"type":"type","doc":"","title":"oidcc_userinfo.error/0","ref":"oidcc_userinfo.html#t:error/0"},{"type":"function","doc":"Load userinfo for the given token\n\nFor a high level interface using `m:oidcc_provider_configuration_worker`, see\n`oidcc:retrieve_userinfo/5`.","title":"oidcc_userinfo.retrieve/3","ref":"oidcc_userinfo.html#retrieve/3"},{"type":"function","doc":"```erlang\n{ok, ClientContext} =\n  oidcc_client_context:from_configuration_worker(provider_name,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>),\n\n%% Get Token\n\n{ok, #{<<\"sub\">> => Sub}} =\n  oidcc_userinfo:retrieve(Token, ClientContext, #{}).\n```","title":"Examples - oidcc_userinfo.retrieve/3","ref":"oidcc_userinfo.html#retrieve/3-examples"},{"type":"type","doc":"Configure userinfo request\n\nSee https://openid.net/specs/openid-connect-core-1_0.html#UserInfoRequest","title":"oidcc_userinfo.retrieve_opts/0","ref":"oidcc_userinfo.html#t:retrieve_opts/0"},{"type":"type","doc":"* `refresh_jwks` - How to handle tokens with an unknown `kid`.\n  See `t:oidcc_jwt_util:refresh_jwks_for_unknown_kid_fun/0`\n* `expected_subject` - expected subject for the userinfo\n  (`sub` from id token)\n* `dpop_nonce` - if using DPoP, the `nonce` value to use in the\n    proof claim","title":"Parameters - oidcc_userinfo.retrieve_opts/0","ref":"oidcc_userinfo.html#t:retrieve_opts/0-parameters"},{"type":"type","doc":"See `t:retrieve_opts/0`.","title":"oidcc_userinfo.retrieve_opts_no_sub/0","ref":"oidcc_userinfo.html#t:retrieve_opts_no_sub/0"},{"type":"module","doc":"OpenID Connect High Level Interface","title":"Oidcc","ref":"Oidcc.html"},{"type":"module","doc":"{:ok, _pid} =\n      Oidcc.ProviderConfiguration.Worker.start_link(%{\n      issuer: \"https://accounts.google.com\",\n      name: MyApp.GoogleConfigProvider\n    })\n\nor via a supervisor\n\n    Supervisor.init([\n      {Oidcc.ProviderConfiguration.Worker, %{issuer: \"https://accounts.google.com\"}}\n    ], strategy: :one_for_one)","title":"Setup - Oidcc","ref":"Oidcc.html#module-setup"},{"type":"module","doc":"* `max_clock_skew` (default `0`) - Maximum allowed clock skew for JWT\n  `exp` / `nbf` validation, in seconds","title":"Global Configuration - Oidcc","ref":"Oidcc.html#module-global-configuration"},{"type":"function","doc":"Retrieve Client Credential Token\n\nSee https://datatracker.ietf.org/doc/html/rfc6749#section-1.3.4","title":"Oidcc.client_credentials_token/4","ref":"Oidcc.html#client_credentials_token/4"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://erlef-test-w4a8z2.zitadel.cloud\"\n    ...>   })\n    ...>\n    ...> {:ok, %Oidcc.Token{}} =\n    ...>   Oidcc.client_credentials_token(\n    ...>     pid,\n    ...>     System.fetch_env!(\"CLIENT_CREDENTIALS_CLIENT_ID\"),\n    ...>     System.fetch_env!(\"CLIENT_CREDENTIALS_CLIENT_SECRET\"),\n    ...>     %{scope: [\"openid\"]}\n    ...>   )","title":"Examples - Oidcc.client_credentials_token/4","ref":"Oidcc.html#client_credentials_token/4-examples"},{"type":"function","doc":"Create Auth Redirect URL","title":"Oidcc.create_redirect_url/4","ref":"Oidcc.html#create_redirect_url/4"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\"\n    ...> })\n    ...>\n    ...> {:ok, _redirect_uri} =\n    ...>   Oidcc.create_redirect_url(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\",\n    ...>     %{redirect_uri: \"https://my.server/return\"}\n    ...>   )","title":"Examples - Oidcc.create_redirect_url/4","ref":"Oidcc.html#create_redirect_url/4-examples"},{"type":"function","doc":"Create Initiate URI for Relaying Party initiated Logout\n\nSee [https://openid.net/specs/openid-connect-rpinitiated-1_0.html#RPLogout]","title":"Oidcc.initiate_logout_url/4","ref":"Oidcc.html#initiate_logout_url/4"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://erlef-test-w4a8z2.zitadel.cloud\"\n    ...>   })\n    ...>\n    ...> # Get access_token from Oidcc.Token.retrieve/3\n    ...> token = \"token\"\n    ...>\n    ...> {:ok, _redirect_uri} = Oidcc.initiate_logout_url(\n    ...>   token,\n    ...>   pid,\n    ...>   \"client_id\"\n    ...> )","title":"Examples - Oidcc.initiate_logout_url/4","ref":"Oidcc.html#initiate_logout_url/4-examples"},{"type":"function","doc":"Introspect the given access token","title":"Oidcc.introspect_token/5","ref":"Oidcc.html#introspect_token/5"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> Oidcc.introspect_token(\n    ...>   \"access_token\",\n    ...>   pid,\n    ...>   \"client_id\",\n    ...>   \"client_secret\"\n    ...> )\n    ...> # => {:ok, %Oidcc.TokenIntrospection{}}","title":"Examples - Oidcc.introspect_token/5","ref":"Oidcc.html#introspect_token/5-examples"},{"type":"function","doc":"Retrieve JSON Web Token (JWT) Profile Token\n\nhttps://datatracker.ietf.org/doc/html/rfc7523#section-4","title":"Oidcc.jwt_profile_token/6","ref":"Oidcc.html#jwt_profile_token/6"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://erlef-test-w4a8z2.zitadel.cloud\"\n    ...>   })\n    ...>\n    ...> %{\"key\" => key, \"keyId\" => kid, \"userId\" => subject} = \"JWT_PROFILE\"\n    ...>   |> System.fetch_env!()\n    ...>   |> JOSE.decode()\n    ...>\n    ...> jwk = JOSE.JWK.from_pem(key)\n    ...>\n    ...> {:ok, %Oidcc.Token{}} =\n    ...>   Oidcc.jwt_profile_token(\n    ...>     subject,\n    ...>     pid,\n    ...>     \"JWT Profile Test\",\n    ...>     \"client_secret\",\n    ...>     jwk,\n    ...>     %{scope: [\"openid\", \"urn:zitadel:iam:org:project:id:zitadel:aud\"], kid: kid}\n    ...>   )","title":"Examples - Oidcc.jwt_profile_token/6","ref":"Oidcc.html#jwt_profile_token/6-examples"},{"type":"function","doc":"Refresh Token","title":"Oidcc.refresh_token/5","ref":"Oidcc.html#refresh_token/5"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> # Get refresh_token from redirect\n    ...> refresh_token = \"refresh_token\"\n    ...>\n    ...> Oidcc.refresh_token(\n    ...>   refresh_token,\n    ...>   pid,\n    ...>   \"client_id\",\n    ...>   \"client_secret\",\n    ...>   %{expected_subject: \"sub_from_initial_id_token\"}\n    ...> )\n    ...> # => {:ok, %Oidcc.Token{}}","title":"Examples - Oidcc.refresh_token/5","ref":"Oidcc.html#refresh_token/5-examples"},{"type":"function","doc":"retrieve the token using the authcode received before and directly validate\nthe result.\n\nthe authcode was sent to the local endpoint by the OpenId Connect provider,\nusing redirects","title":"Oidcc.retrieve_token/5","ref":"Oidcc.html#retrieve_token/5"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> # Get auth_code from redirect\n    ...> auth_code = \"auth_code\"\n    ...>\n    ...> Oidcc.retrieve_token(\n    ...>   auth_code,\n    ...>   pid,\n    ...>   \"client_id\",\n    ...>   \"client_secret\",\n    ...>   %{redirect_uri: \"https://my.server/return\"}\n    ...> )\n    ...> # => {:ok, %Oidcc.Token{}}","title":"Examples - Oidcc.retrieve_token/5","ref":"Oidcc.html#retrieve_token/5-examples"},{"type":"function","doc":"Load userinfo for the given token","title":"Oidcc.retrieve_userinfo/5","ref":"Oidcc.html#retrieve_userinfo/5"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> # Get access_token from Oidcc.Token.retrieve/3\n    ...> access_token = \"access_token\"\n    ...>\n    ...> Oidcc.retrieve_userinfo(\n    ...>   access_token,\n    ...>   pid,\n    ...>   \"client_id\",\n    ...>   \"client_secret\",\n    ...>   %{expected_subject: \"sub\"}\n    ...> )\n    ...> # => {:ok, %{\"sub\" => \"sub\"}}","title":"Examples - Oidcc.retrieve_userinfo/5","ref":"Oidcc.html#retrieve_userinfo/5-examples"},{"type":"module","doc":"Functions to start an OpenID Connect Authorization","title":"Oidcc.Authorization","ref":"Oidcc.Authorization.html"},{"type":"module","doc":"* `[:oidcc, :par_request, :start]`\n  * Description: Emitted at the start of executing a PAR request\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :par_request, :stop]`\n  * Description: Emitted at the end of executing a PAR request\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :par_request, :exception]`\n  * Description: Emitted at the end of executing a PAR request\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`","title":"Telemetry - Oidcc.Authorization","ref":"Oidcc.Authorization.html#module-telemetry"},{"type":"function","doc":"Create Auth Redirect URL\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.create_redirect_url/4`.","title":"Oidcc.Authorization.create_redirect_url/2","ref":"Oidcc.Authorization.html#create_redirect_url/2"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://accounts.google.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> {:ok, _redirect_uri} =\n    ...>   Oidcc.Authorization.create_redirect_url(\n    ...>     client_context,\n    ...>     %{redirect_uri: \"https://my.server/return\"}\n    ...>   )","title":"Examples - Oidcc.Authorization.create_redirect_url/2","ref":"Oidcc.Authorization.html#create_redirect_url/2-examples"},{"type":"module","doc":"Client Configuration for authorization, token exchange and userinfo\n\nFor most projects, it makes sense to use\n`Oidcc.ProviderConfiguration.Worker` and the high-level\ninterface of `Oidcc`. In that case direct usage of this\nmodule is not needed.","title":"Oidcc.ClientContext","ref":"Oidcc.ClientContext.html"},{"type":"function","doc":"Apply OpenID Connect / OAuth2 Profiles to the context\n\nSee `:oidcc_client_context.apply_profiles/2` for more.","title":"Oidcc.ClientContext.apply_profiles/2","ref":"Oidcc.ClientContext.html#apply_profiles/2"},{"type":"function","doc":"iex> {:ok, _pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\",\n    ...>   name: __MODULE__.GoogleConfigProvider\n    ...> })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     __MODULE__.GoogleConfigProvider,\n    ...>     \"client_id\",\n    ...>     \"client_Secret\"\n    ...>   )\n    ...>\n    ...> {:ok, %Oidcc.ClientContext{}, %{}} =\n    ...>   Oidcc.ClientContext.apply_profiles(\n    ...>     client_context,\n    ...>     %{profiles: [:fapi2_message_signing]}\n    ...>   )","title":"Examples - Oidcc.ClientContext.apply_profiles/2","ref":"Oidcc.ClientContext.html#apply_profiles/2-examples"},{"type":"type","doc":"","title":"Oidcc.ClientContext.authenticated_t/0","ref":"Oidcc.ClientContext.html#t:authenticated_t/0"},{"type":"function","doc":"Create Client Context from a `Oidcc.ProviderConfiguration.Worker`","title":"Oidcc.ClientContext.from_configuration_worker/4","ref":"Oidcc.ClientContext.html#from_configuration_worker/4"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\",\n    ...>   name: __MODULE__.GoogleConfigProvider\n    ...> })\n    ...>\n    ...> {:ok, %Oidcc.ClientContext{}} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     __MODULE__.GoogleConfigProvider,\n    ...>     \"client_id\",\n    ...>     \"client_Secret\"\n    ...>   )\n    ...>\n    ...> {:ok, %Oidcc.ClientContext{}} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_Secret\",\n    ...>     %{client_jwks: JOSE.JWK.generate_key(16)}\n    ...>   )","title":"Examples - Oidcc.ClientContext.from_configuration_worker/4","ref":"Oidcc.ClientContext.html#from_configuration_worker/4-examples"},{"type":"function","doc":"Create Client Context manually","title":"Oidcc.ClientContext.from_manual/5","ref":"Oidcc.ClientContext.html#from_manual/5"},{"type":"function","doc":"iex> {:ok, {configuration, _expiry}} =\n    ...>   Oidcc.ProviderConfiguration.load_configuration(\n    ...>     \"https://login.salesforce.com\"\n    ...>   )\n    ...>\n    ...> {:ok, {jwks, _expiry}} =\n    ...>   Oidcc.ProviderConfiguration.load_jwks(\n    ...>     configuration.jwks_uri\n    ...>   )\n    ...>\n    ...> %Oidcc.ClientContext{} =\n    ...>   Oidcc.ClientContext.from_manual(\n    ...>     configuration,\n    ...>     jwks,\n    ...>     \"client_id\",\n    ...>     \"client_Secret\",\n    ...>     %{client_jwks: JOSE.JWK.generate_key(16)}\n    ...>   )","title":"Examples - Oidcc.ClientContext.from_manual/5","ref":"Oidcc.ClientContext.html#from_manual/5-examples"},{"type":"type","doc":"","title":"Oidcc.ClientContext.t/0","ref":"Oidcc.ClientContext.html#t:t/0"},{"type":"type","doc":"","title":"Oidcc.ClientContext.unauthenticated_t/0","ref":"Oidcc.ClientContext.html#t:unauthenticated_t/0"},{"type":"module","doc":"Dynamic Client Registration Utilities","title":"Oidcc.ClientRegistration","ref":"Oidcc.ClientRegistration.html"},{"type":"module","doc":"* `[:oidcc, :register_client, :start]`\n  * Description: Emitted at the start of registering the client\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string()}`\n\n* `[:oidcc, :register_client, :stop]`\n  * Description: Emitted at the end of registering the client\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string()}`\n\n* `[:oidcc, :register_client, :exception]`\n  * Description: Emitted at the end of registering the client\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string()}`","title":"Telemetry - Oidcc.ClientRegistration","ref":"Oidcc.ClientRegistration.html#module-telemetry"},{"type":"function","doc":"Register Client","title":"Oidcc.ClientRegistration.register/3","ref":"Oidcc.ClientRegistration.html#register/3"},{"type":"function","doc":"iex> {:ok, {provider_configuration, _expiry}} =\n    ...>   Oidcc.ProviderConfiguration.load_configuration(\"https://accounts.google.com\")\n    ...>\n    ...> Oidcc.ClientRegistration.register(\n    ...>   provider_configuration,\n    ...>   %Oidcc.ClientRegistration{\n    ...>     redirect_uris: [\"https://your.application.com/oidcc/callback\"]\n    ...>   },\n    ...>   %{initial_access_token: \"optional token you got from the provider\"}\n    ...> )\n    ...> # {:ok, %Oidcc.ClientRegistration.Response{\n    ...> #   client_id: client_id,\n    ...> #   client_secret: client_secret\n    ...> # }}","title":"Examples - Oidcc.ClientRegistration.register/3","ref":"Oidcc.ClientRegistration.html#register/3-examples"},{"type":"type","doc":"Client Metadata Struct\n\nSee https://openid.net/specs/openid-connect-registration-1_0.html#ClientMetadata and\nhttps://openid.net/specs/openid-connect-rpinitiated-1_0.html#ClientMetadata","title":"Oidcc.ClientRegistration.t/0","ref":"Oidcc.ClientRegistration.html#t:t/0"},{"type":"module","doc":"Client Registration Response Struct","title":"Oidcc.ClientRegistration.Response","ref":"Oidcc.ClientRegistration.Response.html"},{"type":"type","doc":"Client Registration Response Struct\n\nSee https://openid.net/specs/openid-connect-registration-1_0.html#RegistrationResponse","title":"Oidcc.ClientRegistration.Response.t/0","ref":"Oidcc.ClientRegistration.Response.html#t:t/0"},{"type":"module","doc":"Logout from the OpenID Provider","title":"Oidcc.Logout","ref":"Oidcc.Logout.html"},{"type":"function","doc":"Initiate URI for Relaying Party initiated Logout\n\nSee https://openid.net/specs/openid-connect-rpinitiated-1_0.html#RPLogout\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.initiate_logout_url/4`.","title":"Oidcc.Logout.initiate_url/3","ref":"Oidcc.Logout.html#initiate_url/3"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://erlef-test-w4a8z2.zitadel.cloud\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     :unauthenticated\n    ...>   )\n    ...>\n    ...> # Get `token` from `Oidcc.retrieve_token/5`\n    ...> token = \"token\"\n    ...>\n    ...> {:ok, _redirect_uri} =\n    ...>   Oidcc.Logout.initiate_url(\n    ...>     token,\n    ...>     client_context,\n    ...>     %{post_logout_redirect_uri: \"https://my.server/return\"}\n    ...>   )","title":"Examples - Oidcc.Logout.initiate_url/3","ref":"Oidcc.Logout.html#initiate_url/3-examples"},{"type":"module","doc":"Tooling to load and parse Openid Configuration","title":"Oidcc.ProviderConfiguration","ref":"Oidcc.ProviderConfiguration.html"},{"type":"module","doc":"* `[:oidcc, :load_configuration, :start]`\n  * Description: Emitted at the start of loading the provider configuration\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string()}`\n\n* `[:oidcc, :load_configuration, :stop]`\n  * Description: Emitted at the end of loading the provider configuration\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string()}`\n\n* `[:oidcc, :load_configuration, :exception]`\n  * Description: Emitted at the end of loading the provider configuration\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string()}`\n\n* `[:oidcc, :load_jwks, :start]`\n  * Description: Emitted at the start of loading the provider jwks\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{jwks_uri: :uri_string.uri_string()}`\n\n* `[:oidcc, :load_jwks, :stop]`\n  * Description: Emitted at the end of loading the provider jwks\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{jwks_uri: :uri_string.uri_string()}`\n\n* `[:oidcc, :load_jwks, :exception]`\n  * Description: Emitted at the end of loading the provider jwks\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{jwks_uri: :uri_string.uri_string()}`","title":"Telemetry - Oidcc.ProviderConfiguration","ref":"Oidcc.ProviderConfiguration.html#module-telemetry"},{"type":"function","doc":"Decode JSON into OpenID configuration","title":"Oidcc.ProviderConfiguration.decode_configuration/2","ref":"Oidcc.ProviderConfiguration.html#decode_configuration/2"},{"type":"function","doc":"iex> {:ok, {{~c\"HTTP/1.1\",200, ~c\"OK\"}, _headers, body}} =\n    ...>   :httpc.request(\"https://accounts.google.com/.well-known/openid-configuration\")\n    ...>\n    ...> decoded_json = body |> to_string() |> JOSE.decode()\n    ...>\n    ...> {:ok, %ProviderConfiguration{issuer: \"https://accounts.google.com\"}} =\n    ...>   Oidcc.ProviderConfiguration.decode_configuration(decoded_json)","title":"Examples - Oidcc.ProviderConfiguration.decode_configuration/2","ref":"Oidcc.ProviderConfiguration.html#decode_configuration/2-examples"},{"type":"function","doc":"Load OpenID Configuration","title":"Oidcc.ProviderConfiguration.load_configuration/2","ref":"Oidcc.ProviderConfiguration.html#load_configuration/2"},{"type":"function","doc":"iex> {:ok, {\n    ...>   %ProviderConfiguration{issuer: \"https://accounts.google.com\"},\n    ...>   _expiry\n    ...> }} = Oidcc.ProviderConfiguration.load_configuration(\"https://accounts.google.com\")","title":"Examples - Oidcc.ProviderConfiguration.load_configuration/2","ref":"Oidcc.ProviderConfiguration.html#load_configuration/2-examples"},{"type":"function","doc":"Load JWKs","title":"Oidcc.ProviderConfiguration.load_jwks/2","ref":"Oidcc.ProviderConfiguration.html#load_jwks/2"},{"type":"function","doc":"iex> {:ok, {%JOSE.JWK{}, _expiry}} =\n    ...>   Oidcc.ProviderConfiguration.load_jwks(\"https://www.googleapis.com/oauth2/v3/certs\")","title":"Examples - Oidcc.ProviderConfiguration.load_jwks/2","ref":"Oidcc.ProviderConfiguration.html#load_jwks/2-examples"},{"type":"type","doc":"Configuration Struct\n\nFor details on the fields see:\n* https://openid.net/specs/openid-connect-discovery-1_0.html#ProviderMetadata\n* https://datatracker.ietf.org/doc/html/draft-jones-oauth-discovery-01#section-4.1\n* https://openid.net/specs/openid-connect-rpinitiated-1_0.html#OPMetadata","title":"Oidcc.ProviderConfiguration.t/0","ref":"Oidcc.ProviderConfiguration.html#t:t/0"},{"type":"module","doc":"OIDC Config Provider Worker\n\nLoads and continuously refreshes the OIDC configuration and JWKs","title":"Oidcc.ProviderConfiguration.Worker","ref":"Oidcc.ProviderConfiguration.Worker.html"},{"type":"module","doc":"```elixir\nSupervisor.init([\n  {Oidcc.ProviderConfiguration.Worker, %{issuer: \"https://accounts.google.com\"}}\n], strategy: :one_for_one)\n```","title":"Usage in Supervisor - Oidcc.ProviderConfiguration.Worker","ref":"Oidcc.ProviderConfiguration.Worker.html#module-usage-in-supervisor"},{"type":"function","doc":"","title":"Oidcc.ProviderConfiguration.Worker.child_spec/1","ref":"Oidcc.ProviderConfiguration.Worker.html#child_spec/1"},{"type":"function","doc":"Get Parsed Jwks","title":"Oidcc.ProviderConfiguration.Worker.get_jwks/1","ref":"Oidcc.ProviderConfiguration.Worker.html#get_jwks/1"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\"\n    ...> })\n    ...> %JOSE.JWK{} =\n    ...>   Oidcc.ProviderConfiguration.Worker.get_jwks(pid)","title":"Examples - Oidcc.ProviderConfiguration.Worker.get_jwks/1","ref":"Oidcc.ProviderConfiguration.Worker.html#get_jwks/1-examples"},{"type":"function","doc":"Get Configuration","title":"Oidcc.ProviderConfiguration.Worker.get_provider_configuration/1","ref":"Oidcc.ProviderConfiguration.Worker.html#get_provider_configuration/1"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\"\n    ...> })\n    ...> %Oidcc.ProviderConfiguration{issuer: \"https://accounts.google.com\"} =\n    ...>   Oidcc.ProviderConfiguration.Worker.get_provider_configuration(pid)","title":"Examples - Oidcc.ProviderConfiguration.Worker.get_provider_configuration/1","ref":"Oidcc.ProviderConfiguration.Worker.html#get_provider_configuration/1-examples"},{"type":"type","doc":"See `t:oidcc_provider_configuration_worker.opts/0`","title":"Oidcc.ProviderConfiguration.Worker.opts/0","ref":"Oidcc.ProviderConfiguration.Worker.html#t:opts/0"},{"type":"function","doc":"Refresh Configuration","title":"Oidcc.ProviderConfiguration.Worker.refresh_configuration/1","ref":"Oidcc.ProviderConfiguration.Worker.html#refresh_configuration/1"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\"\n    ...> })\n    ...> :ok = Oidcc.ProviderConfiguration.Worker.refresh_configuration(pid)","title":"Examples - Oidcc.ProviderConfiguration.Worker.refresh_configuration/1","ref":"Oidcc.ProviderConfiguration.Worker.html#refresh_configuration/1-examples"},{"type":"function","doc":"Refresh JWKs","title":"Oidcc.ProviderConfiguration.Worker.refresh_jwks/1","ref":"Oidcc.ProviderConfiguration.Worker.html#refresh_jwks/1"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\"\n    ...> })\n    ...> :ok = Oidcc.ProviderConfiguration.Worker.refresh_jwks(pid)","title":"Examples - Oidcc.ProviderConfiguration.Worker.refresh_jwks/1","ref":"Oidcc.ProviderConfiguration.Worker.html#refresh_jwks/1-examples"},{"type":"function","doc":"Refresh JWKs if the provided `Kid` is not matching any currently loaded keys","title":"Oidcc.ProviderConfiguration.Worker.refresh_jwks_for_unknown_kid/2","ref":"Oidcc.ProviderConfiguration.Worker.html#refresh_jwks_for_unknown_kid/2"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\"\n    ...> })\n    ...> :ok = Oidcc.ProviderConfiguration.Worker.refresh_jwks_for_unknown_kid(pid, \"kid\")","title":"Examples - Oidcc.ProviderConfiguration.Worker.refresh_jwks_for_unknown_kid/2","ref":"Oidcc.ProviderConfiguration.Worker.html#refresh_jwks_for_unknown_kid/2-examples"},{"type":"function","doc":"Start Configuration Worker","title":"Oidcc.ProviderConfiguration.Worker.start_link/1","ref":"Oidcc.ProviderConfiguration.Worker.html#start_link/1"},{"type":"function","doc":"iex> {:ok, _pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\",\n    ...>   name: __MODULE__.GoogleConfigProvider\n    ...> })","title":"Examples - Oidcc.ProviderConfiguration.Worker.start_link/1","ref":"Oidcc.ProviderConfiguration.Worker.html#start_link/1-examples"},{"type":"module","doc":"Facilitate OpenID Code/Token Exchanges","title":"Oidcc.Token","ref":"Oidcc.Token.html"},{"type":"module","doc":"* `[:oidcc, :request_token, :start]`\n  * Description: Emitted at the start of requesting a code token\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :request_token, :stop]`\n  * Description: Emitted at the end of requesting a code token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :request_token, :exception]`\n  * Description: Emitted at the end of requesting a code token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :refresh_token, :start]`\n  * Description: Emitted at the start of refreshing a token\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :refresh_token, :stop]`\n  * Description: Emitted at the end of refreshing a token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :refresh_token, :exception]`\n  * Description: Emitted at the end of refreshing a token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :jwt_profile_token, :start]`\n  * Description: Emitted at the start of exchanging a JWT profile token\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :jwt_profile_token, :stop]`\n  * Description: Emitted at the end of exchanging a JWT profile token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :jwt_profile_token, :exception]`\n  * Description: Emitted at the end of exchanging a JWT profile token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :client_credentials, :start]`\n  * Description: Emitted at the start of requesting a client credentials token\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :client_credentials, :stop]`\n  * Description: Emitted at the end of requesting a client credentials token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :client_credentials, :exception]`\n  * Description: Emitted at the end of requesting a client credentials token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`","title":"Telemetry - Oidcc.Token","ref":"Oidcc.Token.html#module-telemetry"},{"type":"function","doc":"Retrieve Client Credential Token\n\nSee https://datatracker.ietf.org/doc/html/rfc6749#section-1.3.4\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.client_credentials_token/4`.","title":"Oidcc.Token.client_credentials/2","ref":"Oidcc.Token.html#client_credentials/2"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://erlef-test-w4a8z2.zitadel.cloud\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     System.fetch_env!(\"CLIENT_CREDENTIALS_CLIENT_ID\"),\n    ...>     System.fetch_env!(\"CLIENT_CREDENTIALS_CLIENT_SECRET\")\n    ...>   )\n    ...>\n    ...> {:ok, %Oidcc.Token{}} =\n    ...>   Oidcc.Token.client_credentials(\n    ...>     client_context,\n    ...>     %{scope: [\"openid\"]}\n    ...>   )","title":"Examples - Oidcc.Token.client_credentials/2","ref":"Oidcc.Token.html#client_credentials/2-examples"},{"type":"function","doc":"Retrieve JSON Web Token (JWT) Profile Token\n\nSee https://datatracker.ietf.org/doc/html/rfc7523#section-4\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.jwt_profile_token/6`.","title":"Oidcc.Token.jwt_profile/4","ref":"Oidcc.Token.html#jwt_profile/4"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://erlef-test-w4a8z2.zitadel.cloud\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"JWT Profile Test\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> %{\"key\" => key, \"keyId\" => kid, \"userId\" => subject} = \"JWT_PROFILE\"\n    ...>   |> System.fetch_env!()\n    ...>   |> JOSE.decode()\n    ...>\n    ...> jwk = JOSE.JWK.from_pem(key)\n    ...>\n    ...> {:ok, %Oidcc.Token{}} =\n    ...>   Oidcc.Token.jwt_profile(\n    ...>     subject,\n    ...>     client_context,\n    ...>     jwk,\n    ...>     %{scope: [\"openid\", \"urn:zitadel:iam:org:project:id:zitadel:aud\"], kid: kid}\n    ...>   )","title":"Examples - Oidcc.Token.jwt_profile/4","ref":"Oidcc.Token.html#jwt_profile/4-examples"},{"type":"function","doc":"Refresh Token\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.refresh_token/5`.","title":"Oidcc.Token.refresh/3","ref":"Oidcc.Token.html#refresh/3"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> # Get refresh_token from redirect\n    ...> refresh_token = \"refresh_token\"\n    ...>\n    ...> Oidcc.Token.refresh(\n    ...>   refresh_token,\n    ...>   client_context,\n    ...>   %{expected_subject: \"sub\"}\n    ...> )\n    ...> # => {:ok, %Oidcc.Token{}}","title":"Examples - Oidcc.Token.refresh/3","ref":"Oidcc.Token.html#refresh/3-examples"},{"type":"function","doc":"retrieve the token using the authcode received before and directly validate\nthe result.\n\nthe authcode was sent to the local endpoint by the OpenId Connect provider,\nusing redirects\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.retrieve_token/5`.","title":"Oidcc.Token.retrieve/3","ref":"Oidcc.Token.html#retrieve/3"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> # Get auth_code from redirect\n    ...> auth_code = \"auth_code\"\n    ...>\n    ...> Oidcc.Token.retrieve(\n    ...>   auth_code,\n    ...>   client_context,\n    ...>   %{redirect_uri: \"https://my.server/return\"}\n    ...> )\n    ...> # => {:ok, %Oidcc.Token{}}","title":"Examples - Oidcc.Token.retrieve/3","ref":"Oidcc.Token.html#retrieve/3-examples"},{"type":"type","doc":"","title":"Oidcc.Token.t/0","ref":"Oidcc.Token.html#t:t/0"},{"type":"function","doc":"Validate ID Token\n\nUsually the id token is validated using `retrieve/3`.\nIf you get the token passed from somewhere else, this function can validate it.","title":"Oidcc.Token.validate_id_token/3","ref":"Oidcc.Token.html#validate_id_token/3"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> #Get IdToken from somewhere\n    ...> id_token = \"id_token\"\n    ...>\n    ...> Oidcc.Token.validate_id_token(id_token, client_context, :any)\n    ...> # => {:ok, %{\"sub\" => \"sub\", ... }}","title":"Examples - Oidcc.Token.validate_id_token/3","ref":"Oidcc.Token.html#validate_id_token/3-examples"},{"type":"function","doc":"Validate the JARM response, returning the valid claims as a map.\n\nthe response was sent to the local endpoint by the OpenId Connect provider,\nusing redirects","title":"Oidcc.Token.validate_jarm/3","ref":"Oidcc.Token.html#validate_jarm/3"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> # Get auth_code from redirect\n    ...> response = \"JWT\"\n    ...>\n    ...> Oidcc.Token.validate_jarm(\n    ...>   response,\n    ...>   client_context,\n    ...>   %{}\n    ...> )\n    ...> # => {:ok, %{\"code\" => auth_code}}","title":"Examples - Oidcc.Token.validate_jarm/3","ref":"Oidcc.Token.html#validate_jarm/3-examples"},{"type":"function","doc":"Validate JWT\n\nValidates a generic JWT (such as an access token) from the given provider.\nUseful if the issuer is shared between multiple applications, and the access token\ngenerated for a user at one client is used to validate their access at another client.\n\nValidating an arbitrary JWT token (not an ID token) is not covered by the OpenID\nConnect specification. Therefore the signing / encryption algorithms are not\nderieved from the provider configuration, but must be provided by the caller.","title":"Oidcc.Token.validate_jwt/3","ref":"Oidcc.Token.html#validate_jwt/3"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> # Get JWT from Authorization header\n    ...> jwt = \"jwt\"\n    ...>\n    ...> opts = %{\n    ...>   signing_algs: client_context.provider_configuration.id_token_signing_alg_values_supported\n    ...> }\n    ...>\n    ...> Oidcc.Token.validate_jwt(jwt, client_context, opts)\n    ...> # => {:ok, %{\"sub\" => \"sub\", ... }}","title":"Examples - Oidcc.Token.validate_jwt/3","ref":"Oidcc.Token.html#validate_jwt/3-examples"},{"type":"module","doc":"Access Token struct.\n\nSee `t::oidcc_token.access/0`","title":"Oidcc.Token.Access","ref":"Oidcc.Token.Access.html"},{"type":"function","doc":"Generate a map of authorization headers to use when using the given\n`Oidcc.Token.Access` struct to access an API endpoint.","title":"Oidcc.Token.Access.authorization_headers/5","ref":"Oidcc.Token.Access.html#authorization_headers/5"},{"type":"type","doc":"","title":"Oidcc.Token.Access.t/0","ref":"Oidcc.Token.Access.html#t:t/0"},{"type":"module","doc":"ID Token struct","title":"Oidcc.Token.Id","ref":"Oidcc.Token.Id.html"},{"type":"type","doc":"","title":"Oidcc.Token.Id.t/0","ref":"Oidcc.Token.Id.html#t:t/0"},{"type":"module","doc":"Refresh Token struct","title":"Oidcc.Token.Refresh","ref":"Oidcc.Token.Refresh.html"},{"type":"type","doc":"","title":"Oidcc.Token.Refresh.t/0","ref":"Oidcc.Token.Refresh.html#t:t/0"},{"type":"module","doc":"OAuth Token Introspection\n\nSee https://datatracker.ietf.org/doc/html/rfc7662","title":"Oidcc.TokenIntrospection","ref":"Oidcc.TokenIntrospection.html"},{"type":"module","doc":"* `[:oidcc, :introspect_token, :start]`\n  * Description: Emitted at the start of introspecting the token\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :introspect_token, :stop]`\n  * Description: Emitted at the end of introspecting the token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :introspect_token, :exception]`\n  * Description: Emitted at the end of introspecting the token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`","title":"Telemetry - Oidcc.TokenIntrospection","ref":"Oidcc.TokenIntrospection.html#module-telemetry"},{"type":"function","doc":"Introspect the given access token\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.introspect_token/5`.","title":"Oidcc.TokenIntrospection.introspect/3","ref":"Oidcc.TokenIntrospection.html#introspect/3"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> Oidcc.TokenIntrospection.introspect(\n    ...>   \"access_token\",\n    ...>   client_context\n    ...> )\n    ...> # => {:ok, %Oidcc.TokenIntrospection{}}","title":"Examples - Oidcc.TokenIntrospection.introspect/3","ref":"Oidcc.TokenIntrospection.html#introspect/3-examples"},{"type":"type","doc":"For details on the fields see:\n* https://datatracker.ietf.org/doc/html/rfc7662#section-2.2","title":"Oidcc.TokenIntrospection.t/0","ref":"Oidcc.TokenIntrospection.html#t:t/0"},{"type":"module","doc":"OpenID Connect Userinfo\n\nSee https://openid.net/specs/openid-connect-core-1_0.html#UserInfo","title":"Oidcc.Userinfo","ref":"Oidcc.Userinfo.html"},{"type":"module","doc":"* `[:oidcc, :userinfo, :start]`\n  * Description: Emitted at the start of loading userinfo\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :userinfo, :stop]`\n  * Description: Emitted at the end of loading userinfo\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :userinfo, :exception]`\n  * Description: Emitted at the end of loading userinfo\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`","title":"Telemetry - Oidcc.Userinfo","ref":"Oidcc.Userinfo.html#module-telemetry"},{"type":"function","doc":"Load userinfo for the given token\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.retrieve_userinfo/5`.","title":"Oidcc.Userinfo.retrieve/3","ref":"Oidcc.Userinfo.html#retrieve/3"},{"type":"function","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> # Get access_token from Oidcc.Token.retrieve/3\n    ...> access_token = \"access_token\"\n    ...>\n    ...> Oidcc.Userinfo.retrieve(\n    ...>   access_token,\n    ...>   client_context,\n    ...>   %{expected_subject: \"sub\"}\n    ...> )\n    ...> # => {:ok, %{\"sub\" => \"sub\"}}","title":"Examples - Oidcc.Userinfo.retrieve/3","ref":"Oidcc.Userinfo.html#retrieve/3-examples"},{"type":"extras","doc":"# oidcc\n\nOpenID Connect client library for Erlang.\n\n[![EEF Security WG project](https://img.shields.io/badge/EEF-Security-black)](https://github.com/erlef/security-wg)\n[![Main Branch](https://github.com/erlef/oidcc/actions/workflows/branch_main.yml/badge.svg?branch=main)](https://github.com/erlef/oidcc/actions/workflows/branch_main.yml)\n[![Module Version](https://img.shields.io/hexpm/v/oidcc.svg)](https://hex.pm/packages/oidcc)\n[![Total Download](https://img.shields.io/hexpm/dt/oidcc.svg)](https://hex.pm/packages/oidcc)\n[![License](https://img.shields.io/hexpm/l/oidcc.svg)](https://github.com/erlef/oidcc/blob/main/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/erlef/oidcc.svg)](https://github.com/erlef/oidcc/commits/master)\n[![Coverage Status](https://coveralls.io/repos/github/erlef/oidcc/badge.svg?branch=main)](https://coveralls.io/github/erlef/oidcc?branch=main)\n[![OpenSSF Best Practices](https://www.bestpractices.dev/projects/9602/badge)](https://www.bestpractices.dev/projects/9602)\n[![OpenSSF Scorecard](https://api.scorecard.dev/projects/github.com/erlef/oidcc/badge)](https://scorecard.dev/viewer/?uri=github.com/erlef/oidcc)\n\n \n\n \n   \n   \n   \n \n\nOpenID Certified by [Jonatan Männchen](https://github.com/maennchen) at the\n[Erlang Ecosystem Foundation](https://github.com/erlef) of multiple Relaying\nParty conformance profiles of the OpenID Connect protocol:\nFor details, check the\n[Conformance Test Suite](https://github.com/erlef/oidcc_conformance).\n\n \n\n \n   \n   \n   \n \n\nThe refactoring for `v3` and the certification is funded as an\n[Erlang Ecosystem Foundation](https://erlef.org/) stipend entered by the\n[Security Working Group](https://erlef.org/wg/security).\n\n \n\n\n \n   \n   \n   \n \n\nA security audit was performed by [SAFE-Erlang-Elixir](https://github.com/SAFE-Erlang-Elixir) more info [HERE](https://www.erlang-solutions.com/landings/security-audit-for-erlang-2/).","title":"README","ref":"readme.html"},{"type":"extras","doc":"* [Discovery](https://openid.net/specs/openid-connect-discovery-1_0.html)\n  (`[ISSUER]/.well-known/openid-configuration`)\n* [Client Registration](https://openid.net/specs/openid-connect-registration-1_0.html)\n* Authorization (Code Flow)\n  * [Request Object](https://openid.net/specs/openid-connect-core-1_0.html#RequestObject)\n  * [PKCE](https://oauth.net/2/pkce/)\n  * [Pushed Authorization Requests](https://datatracker.ietf.org/doc/html/rfc9126)\n  * [Authorization Server Issuer Identification](https://datatracker.ietf.org/doc/html/rfc9207)\n* Token\n  * Authorization: `client_secret_basic`, `client_secret_post`,\n    `client_secret_jwt`, and `private_key_jwt`\n  * Grant Types: `authorization_code`, `refresh_token`, `jwt_bearer`, and\n    `client_credentials`\n  * Automatic JWK Refreshing when needed\n* Userinfo\n  * [JWT Response](https://openid.net/specs/openid-connect-core-1_0.html#UserInfoResponse)\n  * [Aggregated and Distributed Claims](https://openid.net/specs/openid-connect-core-1_0.html#AggregatedDistributedClaims)\n* [Token Introspection](https://datatracker.ietf.org/doc/html/rfc7662)\n* Logout\n  * [RP-Initiated](https://openid.net/specs/openid-connect-rpinitiated-1_0.html)\n* [JWT Secured Authorization Response Mode for OAuth 2.0 (JARM)](https://openid.net/specs/oauth-v2-jarm-final.html)\n* [Demonstrating Proof of Possession (DPoP)](https://datatracker.ietf.org/doc/html/rfc9449)\n* [OAuth 2 Purpose Request Parameter](https://cdn.connectid.com.au/specifications/oauth2-purpose-01.html)\n* Profiles\n  * [FAPI 2.0 Security Profile](https://openid.bitbucket.io/fapi/fapi-2_0-security-profile.html)\n  * [FAPI 2.0 Message Signing](https://openid.bitbucket.io/fapi/fapi-2_0-message-signing.html)","title":"Supported Features - README","ref":"readme.html#supported-features"},{"type":"extras","doc":"**Please note that the minimum supported Erlang OTP version is OTP26.**\n\n<!-- tabs-open -->","title":"Setup - README","ref":"readme.html#setup"},{"type":"extras","doc":"**directly**\n\n```erlang\n{ok, Pid} =\n    oidcc_provider_configuration_worker:start_link(#{\n        issuer => <<\"https://accounts.google.com\">>,\n        name => {local, google_config_provider}\n    }).\n```\n\n**via `supervisor`**\n\n```erlang\n-behaviour(supervisor).\n\n%% ...\n\ninit(_Args) ->\n    SupFlags = #{strategy => one_for_one},\n    ChildSpecs = [\n        #{\n            id => oidcc_provider_configuration_worker,\n            start =>\n                {oidcc_provider_configuration_worker, start_link, [\n                    #{\n                        issuer => \"https://accounts.google.com\",\n                        name => {local, myapp_oidcc_config_provider}\n                    }\n                ]},\n            shutdown => brutal_kill\n        }\n    ],\n    {ok, {SupFlags, ChildSpecs}}.\n```","title":"Erlang - README","ref":"readme.html#erlang"},{"type":"extras","doc":"**directly**\n\n```elixir\n{:ok, _pid} =\n  Oidcc.ProviderConfiguration.Worker.start_link(%{\n    issuer: \"https://accounts.google.com\",\n    name: Myapp.OidccConfigProvider\n  })\n```\n\n**via `Supervisor`**\n\n```elixir\nSupervisor.init(\n  [\n    {Oidcc.ProviderConfiguration.Worker,\n     %{\n       issuer: \"https://accounts.google.com\",\n       name: Myapp.OidccConfigProvider\n     }}\n  ],\n  strategy: :one_for_one\n)\n```\n\n**using [`igniter`](https://hex.pm/packages/igniter)**\n\n```bash\nmix oidcc.gen.provider_configuration_worker \\\n  --name MyApp.OidccConfigProvider \\\n  --issuer https://accounts.google.com\n```\n\n<!-- tabs-close -->","title":"Elixir - README","ref":"readme.html#elixir"},{"type":"extras","doc":"<!-- tabs-open -->","title":"Usage - README","ref":"readme.html#usage"},{"type":"extras","doc":"`oidcc` offers integrations for various libraries:\n\n<!-- TODO: Uncomment when available -->\n\n- [`oidcc_cowboy`](https://hex.pm/packages/oidcc_cowboy) - Integrations for\n  [`cowboy`](https://hex.pm/packages/cowboy)\n- [`oidcc_plug`](https://hex.pm/packages/oidcc_plug) - Integrations for\n  [`plug`](https://hex.pm/packages/plug) and\n  [`phoenix`](https://hex.pm/packages/phoenix)\n- [`ueberauth_oidcc`](https://hex.pm/packages/ueberauth_oidcc) - Integration for\n  [`ueberauth`](https://hex.pm/packages/ueberauth)","title":"Companion libraries - README","ref":"readme.html#companion-libraries"},{"type":"extras","doc":"```erlang\n%% Create redirect URI for authorization\n{ok, RedirectUri} = oidcc:create_redirect_url(\n    myapp_oidcc_config_provider,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{redirect_uri => <<\"https://example.com/callback\">>}\n),\n\n%% Redirect user to `RedirectUri`\n\n%% Retrieve `code` query / form param from redirect back\n\n%% Exchange code for token\n{ok, Token} =\n    oidcc:retrieve_token(\n        AuthCode,\n        myapp_oidcc_config_provider,\n        <<\"client_id\">>,\n        <<\"client_secret\">>,\n        #{redirect_uri => <<\"https://example.com/callback\">>}\n    ),\n\n%% Load userinfo for token\n{ok, Claims} =\n    oidcc:retrieve_userinfo(\n        Token,\n        myapp_oidcc_config_provider,\n        <<\"client_id\">>,\n        <<\"client_secret\">>,\n        #{}\n    ),\n\n%% Load introspection for access token\n{ok, Introspection} =\n    oidcc:introspect_token(\n        Token,\n        myapp_oidcc_config_provider,\n        <<\"client_id\">>,\n        <<\"client_secret\">>,\n        #{}\n    ),\n\n%% Refresh token when it expires\n{ok, RefreshedToken} =\n    oidcc:refresh_token(\n        Token,\n        myapp_oidcc_config_provider,\n        <<\"client_id\">>,\n        <<\"client_secret\">>,\n        #{}\n    ).\n```\n\nfor more details, see https://hexdocs.pm/oidcc/oidcc.html","title":"Erlang - README","ref":"readme.html#erlang"},{"type":"extras","doc":"```elixir\n# Create redirect URI for authorization\n{:ok, redirect_uri} =\n  Oidcc.create_redirect_url(\n    Myapp.OidccConfigProvider,\n    \"client_id\",\n    \"client_secret\",\n    %{redirect_uri: \"https://example.com/callback\"}\n  )\n\n# Redirect user to `redirect_uri`\n\n# Retrieve `code` query / form param from redirect back\n\n# Exchange code for token\n{:ok, token} =\n  Oidcc.retrieve_token(\n    auth_code,\n    Myapp.OidccConfigProvider,\n    \"client_id\",\n    \"client_secret\",\n    %{redirect_uri: \"https://example.com/callback\"}\n  )\n\n# Load userinfo for token\n{:ok, claims} =\n  Oidcc.retrieve_userinfo(\n    token,\n    Myapp.OidccConfigProvider,\n    \"client_id\",\n    \"client_secret\",\n    %{expected_subject: \"sub\"}\n  )\n\n# Load introspection for access token\n{:ok, introspection} =\n  Oidcc.introspect_token(\n    token,\n    Myapp.OidccConfigProvider,\n    \"client_id\",\n    \"client_secret\"\n  )\n\n# Refresh token when it expires\n{:ok, refreshed_token} =\n  Oidcc.refresh_token(\n    token,\n    Myapp.OidccConfigProvider,\n    \"client_id\",\n    \"client_secret\"\n  )\n```\n\nfor more details, see https://hexdocs.pm/oidcc/Oidcc.html\n\n<!-- tabs-close -->","title":"Elixir - README","ref":"readme.html#elixir"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}